@@ -15,15 +15,12 @@ from typing import (
     List,
     Mapping,
     Optional,
-    Protocol,
     Sequence,
     Set,
     Tuple,
     Type,
     TypeVar,
     Union,
-    get_args,
-    get_origin,
     no_type_check,
 )
 
@@ -35,6 +32,7 @@ from pydantic.fields import ModelField, Undefined, UndefinedType
 from pydantic.main import ModelMetaclass, validate_model
 from pydantic.typing import NoArgAnyCallable
 from pydantic.utils import Representation
+from typing_extensions import Protocol, get_args, get_origin
 from ulid import ULID
 
 from ..checks import has_redis_json, has_redisearch
@@ -65,6 +63,8 @@ SINGLE_VALUE_TAG_FIELD_SEPARATOR = "|"
 # multi-value field lookup, like a IN or NOT_IN.
 DEFAULT_REDISEARCH_FIELD_SEPARATOR = ","
 
+ERRORS_URL = "https://github.com/redis/redis-om-python/blob/main/docs/errors.md"
+
 
 class RedisModelError(Exception):
     """Raised when a problem exists in the definition of a RedisModel."""
@@ -109,6 +109,7 @@ def embedded(cls):
 
 
 def is_supported_container_type(typ: Optional[type]) -> bool:
+    # TODO: Wait, why don't we support indexing sets?
     if typ == list or typ == tuple:
         return True
     unwrapped = get_origin(typ)
@@ -289,6 +290,11 @@ class ExpressionProxy:
             left=self.field, op=Operators.IN, right=other, parents=self.parents
         )
 
+    def __rshift__(self, other: Any) -> Expression:
+        return Expression(
+            left=self.field, op=Operators.NOT_IN, right=other, parents=self.parents
+        )
+
     def __getattr__(self, item):
         if is_supported_container_type(self.field.outer_type_):
             embedded_cls = get_args(self.field.outer_type_)
@@ -296,7 +302,7 @@ class ExpressionProxy:
                 raise QuerySyntaxError(
                     "In order to query on a list field, you must define "
                     "the contents of the list with a type annotation, like: "
-                    "orders: List[Order]. Docs: TODO"
+                    f"orders: List[Order]. Docs: {ERRORS_URL}#E1"
                 )
             embedded_cls = embedded_cls[0]
             attr = getattr(embedded_cls, item)
@@ -420,7 +426,7 @@ class FindQuery:
             if not getattr(field_proxy.field.field_info, "sortable", False):
                 raise QueryNotSupportedError(
                     f"You tried sort by {field_name}, but {self.model} does "
-                    "not define that field as sortable. See docs: XXX"
+                    f"not define that field as sortable. Docs: {ERRORS_URL}#E2"
                 )
         return sort_fields
 
@@ -434,7 +440,7 @@ class FindQuery:
                 raise QuerySyntaxError(
                     f"You tried to do a full-text search on the field '{field.name}', "
                     f"but the field is not indexed for full-text search. Use the "
-                    f"full_text_search=True option. Docs: TODO"
+                    f"full_text_search=True option. Docs: {ERRORS_URL}#E3"
                 )
             return RediSearchFieldTypes.TEXT
 
@@ -461,7 +467,7 @@ class FindQuery:
         elif container_type is not None:
             raise QuerySyntaxError(
                 "Only lists and tuples are supported for multi-value fields. "
-                "See docs: TODO"
+                f"Docs: {ERRORS_URL}#E4"
             )
         elif any(issubclass(field_type, t) for t in NUMERIC_TYPES):
             # Index numeric Python types as NUMERIC fields, so we can support
@@ -481,8 +487,7 @@ class FindQuery:
         if isinstance(value, str):
             return escaper.escape(value)
         if isinstance(value, bytes):
-            # TODO: We don't decode and then escape bytes objects passed as input.
-            #  Should we?
+            # TODO: We don't decode bytes objects passed as input. Should we?
             # TODO: TAG indexes fail on JSON arrays of numbers -- only strings
             #  are allowed -- what happens if we save an array of bytes?
             return value
@@ -521,8 +526,8 @@ class FindQuery:
             else:
                 raise QueryNotSupportedError(
                     "Only equals (=), not-equals (!=), and like() "
-                    "comparisons are supported for TEXT fields. See "
-                    "docs: TODO."
+                    "comparisons are supported for TEXT fields. "
+                    f"Docs: {ERRORS_URL}#E5"
                 )
         elif field_type is RediSearchFieldTypes.NUMERIC:
             if op is Operators.EQ:
@@ -573,7 +578,6 @@ class FindQuery:
                 value = escaper.escape(value)
                 result += f"-(@{field_name}:{{{value}}})"
             elif op is Operators.IN:
-                # TODO: Implement IN, test this...
                 expanded_value = cls.expand_tag_value(value)
                 result += f"(@{field_name}:{{{expanded_value}}})"
             elif op is Operators.NOT_IN:
@@ -653,13 +657,12 @@ class FindQuery:
             if not field_info or not getattr(field_info, "index", None):
                 raise QueryNotSupportedError(
                     f"You tried to query by a field ({field_name}) "
-                    f"that isn't indexed. See docs: TODO"
+                    f"that isn't indexed. Docs: {ERRORS_URL}#E6"
                 )
         else:
             raise QueryNotSupportedError(
                 "A query expression should start with either a field "
-                "or an expression enclosed in parenthesis. See docs: "
-                "TODO"
+                f"or an expression enclosed in parentheses. Docs: {ERRORS_URL}#E7"
             )
 
         right = expression.right
@@ -672,7 +675,7 @@ class FindQuery:
             else:
                 raise QueryNotSupportedError(
                     "You can only combine two query expressions with"
-                    "AND (&) or OR (|). See docs: TODO"
+                    f"AND (&) or OR (|). Docs: {ERRORS_URL}#E8"
                 )
 
             if isinstance(right, NegatedExpression):
@@ -850,13 +853,14 @@ class FindQuery:
         return result[0]
 
 
-class PrimaryKeyCreator(abc.ABC):
+class PrimaryKeyCreator(Protocol):
     def create_pk(self, *args, **kwargs) -> str:
         """Create a new primary key"""
 
 
 class UlidPrimaryKey:
-    """A client-side generated primary key that follows the ULID spec.
+    """
+    A client-side generated primary key that follows the ULID spec.
     https://github.com/ulid/javascript#specification
     """
 
@@ -967,7 +971,7 @@ class PrimaryKey:
     field: ModelField
 
 
-class BaseMeta(abc.ABC):
+class BaseMeta(Protocol):
     global_key_prefix: str
     model_key_prefix: str
     primary_key_pattern: str
@@ -975,7 +979,6 @@ class BaseMeta(abc.ABC):
     primary_key: PrimaryKey
     primary_key_creator_cls: Type[PrimaryKeyCreator]
     index_name: str
-    abstract: bool
     embedded: bool
     encoding: str
 
@@ -995,7 +998,6 @@ class DefaultMeta:
     primary_key: Optional[PrimaryKey] = None
     primary_key_creator_cls: Optional[Type[PrimaryKeyCreator]] = None
     index_name: Optional[str] = None
-    abstract: Optional[bool] = False
     embedded: Optional[bool] = False
     encoding: str = "utf-8"
 
@@ -1270,17 +1272,23 @@ class HashModel(RedisModel, abc.ABC):
         super().__init_subclass__(**kwargs)
 
         for name, field in cls.__fields__.items():
+            origin = get_origin(field.outer_type_)
+            if origin:
+                for typ in (Set, Mapping, List):
+                    if issubclass(origin, typ):
+                        raise RedisModelError(
+                            f"HashModels cannot index set, list,"
+                            f" or mapping fields. Field: {name}"
+                        )
+
             if issubclass(field.outer_type_, RedisModel):
                 raise RedisModelError(
-                    f"HashModels cannot have embedded model " f"fields. Field: {name}"
+                    f"HashModels cannot index embedded model fields. Field: {name}"
+                )
+            elif dataclasses.is_dataclass(field.outer_type_):
+                raise RedisModelError(
+                    f"HashModels cannot index dataclass fields. Field: {name}"
                 )
-
-            for typ in (Set, Mapping, List):
-                if issubclass(field.outer_type_, typ):
-                    raise RedisModelError(
-                        f"HashModels cannot have set, list,"
-                        f" or mapping fields. Field: {name}"
-                    )
 
     def save(self, pipeline: Optional[Pipeline] = None) -> "HashModel":
         self.check()
@@ -1361,6 +1369,8 @@ class HashModel(RedisModel, abc.ABC):
         for name, field in cls.__fields__.items():
             # TODO: Merge this code with schema_for_type()?
             _type = field.outer_type_
+            is_subscripted_type = get_origin(_type)
+
             if getattr(field.field_info, "primary_key", None):
                 if issubclass(_type, str):
                     redisearch_field = (
@@ -1373,7 +1383,12 @@ class HashModel(RedisModel, abc.ABC):
                 schema_parts.append(redisearch_field)
             elif getattr(field.field_info, "index", None) is True:
                 schema_parts.append(cls.schema_for_type(name, _type, field.field_info))
-            elif is_supported_container_type(_type):
+            elif is_subscripted_type:
+                # Ignore subscripted types (usually containers!) that we don't
+                # support, for the purposes of indexing.
+                if not is_supported_container_type(_type):
+                    continue
+
                 embedded_cls = get_args(_type)
                 if not embedded_cls:
                     # TODO: Test if this can really happen.
